@@ -5,6 +5,7 @@ import torch
 import torch.nn as nn
 from torch.utils.data import DataLoader
 from torch.optim import Optimizer
+import matplotlib.pyplot as plt
 
 
 class Module(ABC, nn.Module):
@@ -235,3 +236,53 @@ class Trainer:
                     break
         return torch.load(self.checkpoint_final)
 
+    def debug_lr(self, n_iters: int, batch_debug: list, lr_candidates: list = [5e-1, 1e-1, 5e-3, 1e-3, 5e-4, 1e-4, 5e-5, 1e-5, 5e-6, 1e-6], fig_name: str = "lr_debug"):
+        """
+
+        :param lr_candidates:
+        :param n_iters:
+        :param batch_debug:
+        :return:
+        """
+        plt.figure(figsize=(30, 15))
+        plt.suptitle("overfitting one batch with various learning rates")
+        for i, cand in enumerate(lr_candidates):
+            mimo = torch.load(self.checkpoint_initial)
+            adam = torch.optim.Adam(params=mimo.parameters(), lr=cand)
+            _, losses_debug = self.overfit_one_train_batch(module=mimo,
+                                                           batch=batch_debug,
+                                                           optimizer=adam,
+                                                           n_iters=n_iters,
+                                                           freeze_pretrained_layers=False)
+            plt.subplot(2, 5, i + 1)
+            plt.title("lr: " + str(cand))
+            plt.ylabel("loss")
+            plt.xlabel("training iteration")
+            plt.plot(torch.arange(len(losses_debug)), losses_debug)
+        plt.tight_layout()
+        plt.savefig(f"../monitoring/{fig_name}.png")
+
+    def train_n_epochs_initial_lrrt(self, n_epochs: int, freeze_pretrained_layers: bool):
+        """
+        determines the initial learning rate per epoch using lrrt.
+
+        :param int n_epochs: #training epochs after determining the initial learning rate with lrrt
+        :param bool freeze_pretrained_layers:
+        :return: trained module
+        """
+        module = torch.load(self.checkpoint_running)
+        loss_train, loss_val_last = self.losses_epoch_eval(module=module)
+        print("initial eval loss val", loss_val_last, "initial eval loss train", loss_train)
+        for epoch in range(1, n_epochs + 1):
+            print("training epoch", epoch)
+            lr_best, _ = self.lrrt(freeze_pretrained_layers=freeze_pretrained_layers)
+            module = torch.load(self.checkpoint_running).to(self.device)
+            optimizer = self.optimizer_class(params=module.parameters(), lr=lr_best)
+            self.train_n_batches(module=module, optimizer=optimizer, n_batches=len(self.loader_train),
+                                 freeze_pretrained_layers=freeze_pretrained_layers)
+
+            loss_train, loss_val = self.losses_epoch_eval(module=module)
+            print("eval loss val", loss_val, "eval loss train", loss_train)
+            torch.save(module, self.checkpoint_running)
+            torch.save(module, self.checkpoint_final)
+        return torch.load(self.checkpoint_final)
